function plotperf(self, varargin)
    % PLOTPERF Plot and compare rotor performance.
    %   This method plots and compares the performance of the rotor under various operating
    %   conditions and for various solvers.
    %
    % -----
    %
    % Syntax:
    %   Result.plotperf(perf, varOper, alt, rpm, coll, speed) Plot the rotor performance `perf`
    %   with respect to `varOper` for the operating point defined by `alt`, `rpm`, `coll`, `speed`.
    %
    %   Result.plotperf(..., 'solvers', SolversList) Plots only the line for the solvers specified
    %   in `SolversList`. By default all solvers are used.
    %
    %   Result.plotperf(..., 'newFig', true) Create the plot in a new figure.
    %
    % Inputs:
    %   perf    : Performance metric to plot ('cT', 'cP', 'cQ')
    %   varOper : Operational points to plot against ('altitude', 'speed', 'rpm', 'collective')
    %   alt     : Altitude, [m]
    %   rpm     : Rotational speed, [rpm]
    %   coll    : Collective pitch, [deg]
    %   speed   : Axial velocity, [m/s]
    %
    % Outputs:
    %   plot(s) : Plots with the rotor performance.
    %
    % Example:
    %   ResRot.plotperf('cT', 'rpm', 0, 0, nan, 5, 'solvers', {'leishman'})
    %   ResRot.plotperf('cT', 'collective', 0, 10, 200, 5, 'solvers', {'leishman','indvel'})
    %
    % See also: rotare, Result, template.
    %
    % <a href="https://gitlab.uliege.be/rotare/documentation">Complete documentation (online)</a>

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/rotare/rotare
    % Docs: https://gitlab.uliege.be/rotare/documentation
    % Issues: https://gitlab.uliege.be/rotare/rotare/-/issues
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    DEF.ALLOWED_PROPERTIES = {'thrust', 'torque', 'power', 'eff', 'cT', 'cQ', 'cP'};
    DEF.base_oper = self.operPts.Properties.VariableNames;
    DEF.allowed_oper = [DEF.base_oper, 'advRat']; % Also allows advance ratio
    DEF.ALLOWED_SOLVERS = {'leishman', 'indfact', 'indvel', 'stahlhut'};
    DEF.LEISHMAN_LINE = {'LineStyle', '-', 'color', 'red'};
    DEF.INDFACT_LINE = {'LineStyle', ':', 'color', 'magenta'};
    DEF.INDVEL_LINE = {'LineStyle', '--', 'color', 'blue'};
    DEF.STAHLHUT_LINE = {'LineStyle', '-.', 'color', 'green'};

    % Parse inputs
    [perf, oper, solvers, operPt, newFig, lineSpec] = parseinputs(DEF, varargin{:});

    % Plot the figure
    if newFig
        figure('Name', 'Rotor performance');
    end

    [filtered, idx] = self.filter(operPt.altitude, operPt.speed, operPt.rpm, operPt.collective);

    for iSolv = 1:length(solvers)
        thissolv = solvers{iSolv};
        if ~isempty(self.(thissolv))
            if strcmpi(oper, 'advRat')
                operVect = vectorize(filtered.(thissolv), 'advanceRatio');
            else
                operVect = self.operPts(idx, :).(oper);
            end
            dataVect = vectorize(filtered.(thissolv), perf);

            plot(operVect, dataVect, lineSpec.(thissolv){:}, 'DisplayName', thissolv);
            hold on;
        end
    end

end

function [perf, oper, solvers, operPt, newFig, lineSpec] = parseinputs(DEF, varargin)
    % PARSEINPUTS Parse the varargin inputs
    %   - Only the parameter corresponding to oper can be nan, all others need to be defined

    valData = @(x) validateattributes(x, {'numeric'}, {'scalar'});
    valLogi = @(x) validateattributes(x, {'logical'}, {'scalar'});

    p = inputParser;
    addRequired(p, 'perf', @(x) any(validatestring(x, DEF.ALLOWED_PROPERTIES)));
    addRequired(p, 'oper', @(x) any(validatestring(x, DEF.allowed_oper)));
    addRequired(p, 'altitude', valData);
    addRequired(p, 'speed', valData);
    addRequired(p, 'rpm', valData);
    addRequired(p, 'collective', valData);
    addParameter(p, 'solvers', DEF.ALLOWED_SOLVERS);
    addParameter(p, 'newFig', false, valLogi);
    addParameter(p, 'leishmanLine', DEF.LEISHMAN_LINE);
    addParameter(p, 'indfactLine', DEF.INDFACT_LINE);
    addParameter(p, 'indvelLine', DEF.INDVEL_LINE);
    addParameter(p, 'stahlhutLine', DEF.STAHLHUT_LINE);

    parse(p, varargin{:});
    perf = p.Results.perf;
    oper = p.Results.oper;
    solvers = p.Results.solvers;
    operPt = struct('altitude', [], 'speed', [], 'rpm', [], 'collective', []);
    for i = 1:length(DEF.base_oper)
        curOp = DEF.base_oper{i};
        operPt.(curOp) = p.Results.(curOp);
    end
    if strcmpi(p.Results.oper, 'advRat')
        operPt.rpm = NaN;
        operPt.speed = NaN;
    else
        operPt.(p.Results.oper) = NaN;
    end
    newFig = p.Results.newFig;

    lineSpec.leishman = p.Results.leishmanLine;
    lineSpec.indfact = p.Results.indfactLine;
    lineSpec.indvel = p.Results.indvelLine;
    lineSpec.stahlhut = p.Results.stahlhutLine;

end

function vect = vectorize(struct, prop)
    % VECTORIZE Transform struct array into a vector

    vect = zeros(1, length(struct));

    for i = 1:length(struct)
        vect(i) = struct(i).(prop);
    end

end
